package dev.adriankuta.pixabay.feature.details

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.assisted.Assisted
import dagger.assisted.AssistedInject
import dagger.hilt.android.lifecycle.HiltViewModel
import dev.adriankuta.pixabay.data.Result
import dev.adriankuta.pixabay.data.repository.ImageRepository
import dev.adriankuta.pixabay.feature.details.di.PhotoDetailsViewModelFactory
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn

@HiltViewModel(assistedFactory = PhotoDetailsViewModelFactory::class)
class PhotoDetailViewModel @AssistedInject constructor(
    private val imageRepository: ImageRepository,
    @Assisted private val imageId: Int
) : ViewModel() {

    private val loadedData = flow {
        emit(imageRepository.searchImageById(imageId))
    }

    val uiState = loadedData
        .map {
            when (it) {
                is Result.Success -> PhotoDetailUiState.Success(it.data)
                is Result.Error -> PhotoDetailUiState.Error
            }
        }
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.WhileSubscribed(stopTimeoutMillis = 5000),
            initialValue = PhotoDetailUiState.Loading
        )
}